/* Ck: C program source for the File Encryption Signature module.
 * 
 * This program represents the core of the SPI/UNIX package integrity scheme.
 * It employs the RSA Data Security, Inc. MD5 Message-Digest Algorithm.
 * It generates a 128 bit MD5 encryption signature, in the form of a 32 hexchar
 * string.
 *
 * Usage:
 *
 *	ck input_file     (or)     cat input_file | ck
 *
 * Ck reads from stdin or from the file specified on the command line.  
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#ifdef FILEH 
# ifdef FILEOK
#  include FILEH
# else
#  ifdef FCNTLH
#   ifdef FCNTLOK
#    include FCNTLH
#   endif
#  endif 
# endif
#else
# ifdef FCNTLH
#  ifdef FCNTLOK
#   include FCNTLH
#  endif
# endif 
#endif

main(argc,argv)
int argc;
char *argv[];
	{
	extern char * malloc();
	extern char * calc_MD();
	char * infile;		/* name of input file if specified */
	int file;		/* flag: file specified on command line */
	struct stat filestat;	/* status of a file */
	char *newmd;		/* MD5 signature */

	/* Do command line arguments */

	file = 0;

	if ( argc > 2 )
		fprintf (stderr,
			"Too many arguments, trailing arguments ignored ...\n");
	if ( argc < 2 )
		{
		fprintf (stderr,"Usage: ck file  (or)  cat file | ck\n");
		exit (2);
		}
	if ( argc == 2 )
		{
		infile = malloc (strlen (argv[1]) + 1);
		strcpy (infile,argv[1]);
		file++;
		}
	

	/* set up input file */
	if (file)
		if ( freopen(infile,"r",stdin) == NULL)
			{
			fprintf(stderr, "Unable to open input file: %s\n",
				infile);
			exit(1);
			}

	newmd = NULL;
	if ( stat(infile,&filestat) < 0 )
	{
		/* file not available */
		fprintf (stderr,"Can't stat inode for file %s",
			infile);
		fprintf (stdout,"%s\n",infile);
		exit( 2 );
	}
	if ( filestat.st_mode & (S_IFDIR|S_IFREG) ) 
	{
		newmd = calc_MD(infile);
		if( newmd != NULL )
		{
			fprintf(stdout,"%s\n",newmd);
		}
	}
	else
	{
		/* file not dir or regular */
		fprintf (stderr,"File not proper type: %s",
			infile);
		fprintf (stdout,"%s\n",infile);
	}
}
